# 🚀 programmering.no | 🤓 matematikk.as
# - Regn ut summen av elementene i en liste med for-løkke

# Verdier
liste = [2, 3, 10]

# Funksjon som summerer el i en liste
def sum_liste(liste):

	# Nullstill
	sum = 0

	# Oppdater sum for hver iterasjon
	for el in liste:
		sum += el

	return(sum)

# Sum
sum = sum_liste(liste)

# Print
print(f"Liste                : {liste}")
print(f"Summen av elementene : {sum}")



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Torodd)
# Oppgave 6 b) Tredjegradsfunksjon - Påstand 2: Alle linjer på formen y = ax + b vil skjære grafen til f 

from sympy import symbols, limit, oo

# definerer funksjonen og symbola
x, a, b, c, d = symbols("x a b c d")
f = a*x**3 + b*x**2 + c*x + d

# grense mot uendeleg
limit(f, x, oo)

# grense mot minus uendeleg
løsning = limit(f, x, -oo)
print(løsning)



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 1 c) og d) Etterspørsel - SPEEDRUN 
# - Løser oppgaven med CAS i Python
# - Kopier denne kommandoen i terminalen for å importere matematikk: 
#   $ pip install matematikk

import matematikk as mt

_blokk = 1

# Oppg c) - pris_og_enheter_fra_inntekt_max
if _blokk == 1:

    # Antall enheter (etterspørsel) og pris per enhet når inntekten er størst
    enhet_og_pris_ls = mt.enhet_og_pris_fra_inntekt_max(pris_uttrykk_hs = 79 - 12.2 * mt.ln(mt.Symbol("x")),
                                                        pris_desimal    = 2,
                                                        enhet_vari      = mt.Symbol("x"),
                                                        enhet_desimal   = None,
                                                        enhet_debug     = -1)

    # Svar-setninger
    print(f""); print(f"Oppg c)"); print(f"")
    print(f"- Inntekten er størst når etterspørselen er ca. {enhet_og_pris_ls[0]} enheter")
    print(f"- Dette gir en pris på ca. {enhet_og_pris_ls[1]} kr")

# Oppg d)
if _blokk == 1:

    # Optimalt antall enheter (ettspørsel) når overskuddet er størst
    x_opt = mt.enhet_fra_overskudd_max(kostnad_uttrykk_hs = 0.021 * mt.Symbol("x")**2 + 10 * mt.Symbol("x") + 910,
                                       enhet_vari         = mt.Symbol("x"),
                                       enhet_desimal      = None,
                                       enhet_debug        = -1)

    # Svar-setninger 
    print(f""); print(f"Oppg d)"); print(f"")
    print(f"- Grense-inntekten er lik grense-kostnaden ved {x_opt} enheter")
    print(f"- Dette betyr at {x_opt} enheter gir størst overskudd fordi:")
    print(f"     O`(x) = 0, max overskudd")
    print(f"     O`(x) = I`(x) - K`(x), definisjonen av overskudd")
    print(f"     0 = I`(x) - K`(x) <=> K`(x) = I`(x)")




# 🚀 programmering.no | 🤓 matematikk.as
# - Reverserer rekkefølgen til elementene i en liste
# - Hastighet reverse(): Ca. 33 ms per loop for liste m/ 100 000 000 el

# Verdier
original_liste = ["Python", "JavaScript", "Swift", "C++"]

# Kopier listen hvis du vil beholde originalen
reversert_liste = original_liste.copy()

# Reverse
reversert_liste.reverse()

# Print
print(f"Original liste  : {original_liste}")
print(f"Reversert liste : {reversert_liste}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Se om intervallet har primtall

from math import sqrt

# Input
min = input("Min verdi: ")
max = input("Max verdi: ")

# Funksjon for å se intervallet har primtall
def er_primtall_intervall_sqrt(min, max):

    # Itererer gjennom intervallet
	for n in range(int(min), int(max) + 1):

		# Alle primtall er større enn 1
		if n > 1:

			# Iterer fra 2 til sqrt(n) + 1 (raskest tempo)
			for i in range(2, int(sqrt(n)) + 1):

				# n er ikke et primtall hvis det er delelig med i
				if n % i == 0:
					print(f"Er {n} et primtall? Nei")
					break

			# Hvis for-løkken fullføres uten break
			else: print(f"Er {n} et primtall? Ja")

		# Hvis tallet ikke er større enn 1 er det ikke primtall
		else: print(f"Er {n} et primtall? Nei")

# Primtall eller ikke
er_primtall_intervall_sqrt(min, max)



# 🚀 programmering.no | 🤓 matematikk.as
# Sinus 1T (2020) - BrettBoka - 1.5 Formler (s. 31) 
# Oppgave 1.59 Vindpark I (Matematikk AS)

# Blokk med definisjoner
_blokk             = 1 # 0: Av, 1: På, Skjul/debug deler av koden med if-blokker
if _blokk == 1:

    # Konstanter
    pi                 = 3.14          # Pi avrundet til 3.14
    turbin_radius      = 58.5          # Rotorbladets lengde, [m]
    turbin_areal       = 0             # Turbinens areal, [m^2], A = pi * r^2
    turbin_k           = 0.0003        # Turbin-konstant (lufttetthet og virkningsgrad), [W(s^3/m^5)]
    turbin_effekt      = 0             # Turbinens effekt (sekund), [kW]
    turbin_energi      = 0             # Turbinens energi (år), [kWh]
    turbin_vindpark    = 80            # Antall turbiner i vindparken
    vindstyrke         = 5             # Lett bris, [m/s]
    tid_h_per_dogn     = 24            # Timer per døgn, [h/døgn]
    tid_dogn           = 300           # Døgn turbinen kjører, [døgn]
    tid_h              = 0             # Timer turbinen kjører, [h]
    hus_energi         = 20000         # Energi per hus (år), [kWh]
    hus_turbin         = 0             # Antall hus en turbin forsyner (år)
    hus_vindpark       = 0             # Antall hus vindparken forsyner (år)

    # Enheter
    enhet_m_kvad       = "m^2"         # Kvadratmeter
    enhet_kw           = "kW"          # Kilowatt
    enhet_kwh          = "kWh"         # Kilowattimer

    # Tekst
    txt_oppg           = "Oppg 1.59"
    txt_strek          = "---------"
    txt_a              = "a) Turbinens areal                     :"
    txt_b              = "b) Turbinens effekt (sekund)           :"
    txt_c              = "c) Turbinens energi (år)               :"
    txt_d              = "d) Antall hus en turbin forsyner (år)  :"
    txt_e              = "e) Antall hus vindparken forsyner (år) :"

    # Innstillinger
    inp                = 0             # # 0: Boken, 1: Bruker
    avrund_valg        = 1             # # 0: Ikke avrunding, 1: Avrunding
    avrund_ant         = None          # # None: 12.78 -> 13, 0: 12.78 -> 13.0, 1: 12.78 -> 12.8, 2: 12.78 -> 12.78

# Input
print("")
print(txt_strek)
print(txt_oppg)
print(txt_strek)
# if inp == 1:
#     print("")
#     turbin_radius = input("Hvor lange er rotorbladene [m]? ")
#     vindstyrke = input("Hva er vindstyrken [m/s]? ")
#
#     # Type casting: str -> float
#     turbin_radius      = float(turbin_radius)
#     vindstyrke         = float(vindstyrke)

# a) Turbinens areal
turbin_areal       = pi * (turbin_radius**2)

# b) Turbinens effekt (sekund)
turbin_effekt      = turbin_k * (vindstyrke**3) * turbin_areal

# c) Turbinens energi (år)
tid_h              = tid_h_per_dogn * tid_dogn
turbin_energi      = turbin_effekt * tid_h

# d) Antall hus en turbin forsyner (år)
hus_turbin         = turbin_energi / hus_energi

# e) Antall hus vindparken forsyner (år)
hus_vindpark       = hus_turbin * turbin_vindpark

# Avrunding
if avrund_valg == 1:
    turbin_areal   = round(turbin_areal, avrund_ant)
    turbin_effekt  = round(turbin_effekt, avrund_ant)
    turbin_energi  = round(turbin_energi, avrund_ant)
    hus_turbin     = round(hus_turbin, avrund_ant)
    hus_vindpark   = round(hus_vindpark, avrund_ant)

# Print
print("")
print(txt_a, turbin_areal, enhet_m_kvad)
print(txt_b, turbin_effekt, enhet_kw)
print(txt_c, turbin_energi, enhet_kwh)
print(txt_d, hus_turbin)
print(txt_e, hus_vindpark)
print("")



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 1 a) Sofa-produksjon til møbelfabrikk - Finn O(x) 
# - Løser oppgaven med CAS i Python
# - Lager polynom-funksjon av valgfri grad vha. regresjon (CAS)
# - Kopier denne kommandoen i terminalen for å importere matematikk (CAS)
#   $ pip install matematikk

from matematikk import reggis, Symbol

# Variabler
x                  = Symbol("x")       # Polynomets variabel
K                  = Symbol("K")       # Polynomets navn (vs)
grad               = 2                 # Polynomets grad
x_liste            = [                 # Liste med x-verdier
    10,
    25,
    40,
    70,
    100,
    140,
    180]
y_liste            = [                 # Liste med y-verdier
    270,
    550,
    870,
    1500,
    2200,
    3300,
    4500]
rund               = 2                 # -1: Ingen avrunding, 0: Eks: 123.0, n > 0: n siffer etter desimaltegnet, None: Heltall
_blokk             = 1                 # 0: Av, 1: På, Skjul/debug deler av koden med if-blokker

# a) Finn O(x)
if _blokk == 1:

    # Lager polynom-funksjon av valgfri grad vha. regresjon (string)
    # - Lager polynom-funksjon av valgfri grad vha. regresjon (CAS)
    K = reggis(variabel = x,
               grad     = grad,
               x_liste  = x_liste,
               y_liste  = y_liste,
               rund     = rund)

    # Svar-setninger
    svar_a_liste = list()
    svar_a_liste.append(f"")
    svar_a_liste.append(f"Oppg a)")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"- Med regresjon fra numpy (polyfit) ser vi at hvis bedriften")
    svar_a_liste.append(f"  produserer x enheter, så vil")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"    {K}")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"  være en god model for det måntlige overskuffet (i tusen kroner)")

    # Print svar-setninger
    for svar in svar_a_liste: print(svar)



import matematikk as mt

def enhet_fra_overskudd_max_med_kostnad(kostnad_uttrykk_hs = mt.Symbol("3*x + 4"),
                            pris_uttrykk_hs    = mt.Symbol("2*x + 1"),
                            enhet_vari         = mt.Symbol("x"),
                            enhet_desimal      = None,
                            enhet_debug        = -1):

    # Variabler
    x = enhet_vari
    p = pris_uttrykk_hs
    K = kostnad_uttrykk_hs

    # Definerer uttrykket for inntekts-funksjonen, I(x)
    I = x * p

    # Deriverer I mhp. x og får dI = 66.8 - 12.2*log(x)
    dI = mt.deriver(I, x) # I'(x)

    # Deriverer K mhp. x og får dK = 0.042*x + 10
    dK = mt.deriver(K, x) # K'(x)

    # Løser likningen dI = dK mhp. x
    x_opt = mt.superløs(variabel = x,
                        vs       = dI,
                        hs       = dK,
                        rund     = enhet_desimal,
                        debug    = enhet_debug)

    return x_opt
 
# Alias > 1 ...
enhet_fra_overskudd_max_kostnad     = enhet_fra_overskudd_max_med_kostnad

# Alias > 2 > ...
enhet_fra_max_overskudd_med_kostnad = enhet_fra_overskudd_max_med_kostnad
enhet_fra_max_overskudd_kostnad     = enhet_fra_overskudd_max_med_kostnad



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (NDLA)
# Oppgave 5 a) Karakter-statestikk - Normalfordelt karakter-snitt til 20 elever fra 3 skoler 

import numpy as np

E = []     #liste for elevenes karakterer

for i in range(20):
    s = np.random.randint(3) # trekker en av tre skoler

if s == 0:
    E.append(np.random.normal(3.8,1.2))
elif s == 1:
    E.append(np.random.normal(3.4,1.4))
else:
    E.append(np.random.normal(4.1,1.1))
print(f"Gjennomsnittskarakteren er {np.mean(E):.1f}.")



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 1 c) Etterspørsel - Størst inntekt 
# - Løser oppgaven med CAS i Python
# - Kopier denne kommandoen i terminalen for å importere sympy: 
#   $ pip install sympy

import sympy as sp

# Variabler
x                   = sp.Symbol("x")        # Etterspørsel (antall produserte enheter)
p                   = sp.Symbol("p")        # Pris per enhet
I                   = sp.Symbol("I")        # Inntekt
svar_c_ls           = list()                # Svar-setninger
desimal             = 1                     # Avrunding

# Definerer uttrykket for pris-funksjonen, p(x)
p = 79 - 12.2 * sp.ln(x)

# Definerer uttrykket for inntekts-funksjonen, I(x)
I = x * p

# Deriverer I mhp. x og får dI = 66.8 - 12.2*log(x)
dI = sp.diff(I, x) # I'(x)

# Lager likningen dI = 0 med sp.Eq(vs, hs)
likning = sp.Eq(dI, 0)

# Løser likningen for x med solveset() og får løsnings-settet {238.748294544347}
x_inntekt_max_set = sp.solveset(likning, x, domain=sp.Reals) # Løsnings-domene i ℝ (Reals)

# Henter elementet i fra løsnings-settet, 238.748294544347
x_inntekt_max = x_inntekt_max_set.args[0]

# Runder av 238.748294544347 -> 239
x_inntekt_max = round(x_inntekt_max, None)

# Setter x_inntekt_max = 239 inn i p og definerer det nye uttrykket som p_inntekt_max
p_inntekt_max = p.subs(x, x_inntekt_max)

# Henter verdien til p_inntekt_max
p_inntekt_max_val = p_inntekt_max.evalf()

# Runder av 12.1871446664356 -> 12.20
p_inntekt_max_val = round(p_inntekt_max_val, desimal)

# Svar-setninger
svar_c_ls.append(f"")
svar_c_ls.append(f"Oppg c)")
svar_c_ls.append(f"")
svar_c_ls.append(f"- Inntekten er størst når etterspørselen er ca. {x_inntekt_max} enheter")
svar_c_ls.append(f"- Dette gir en pris på ca. {p_inntekt_max_val} kr")

# Print svar-setninger
for svar in svar_c_ls: print(svar)



# 🚀 programmering.no | 🤓 matematikk.as
# - Regn ut summen av de n første kubene i rekken

# Verdier
n = 12

# Sum-funksjon for n kuber
def rekke_n_kuber_sum(n):

    # Formel for sum av n kuber
    sum = int((n * (n + 1) / 2)**2)

    return sum

# Summer n kuber
sum = rekke_n_kuber_sum(n)

# Print
print(f"Summen av de {n} første kubene er: {sum}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Fjern alle positive tall fra en liste med remove()

# Input
liste = input("Liste: ") # Husk [ ] og "", f.eks. [2.5, 10]

# Enkel funksjon som klipper opp inp-str til liste med float-el
def str_til_float_liste(str_liste = list()):

    float_liste = list()
    el = str()
    for i in range(len(str_liste)):
        if str_liste[i] == " ": pass
        elif str_liste[i] == "[": pass
        elif str_liste[i] == "]":
            float_liste.append(float(el))
            el = ""
        elif str_liste[i] == ",":
            float_liste.append(float(el))
            el = ""
        else: el += str_liste[i]
    return float_liste

# Funksjon som fjerner positive tall fra en liste
def fjern_positive_tall(liste):

	# Nullstill
	negative_tall_liste = list()

	# Itererer gjennom listen
	for tall in liste:

		# Legg tallet i listen hvis det er negativt
		if tall < 0:
			negative_tall_liste.append(tall)

	return negative_tall_liste

# Input-string endres til liste med float-el
liste = str_til_float_liste(liste)

# Liste med positive tall
negative_tall_liste = fjern_positive_tall(liste)

# Print
print(f"Original liste          : {liste}")
print(f"Liste med negative tall : {negative_tall_liste}")



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (Udir)
# Oppgave 3 a) Aritmetisk rekke - Sum av n = 10 ledd 

a = 3
d = 4

N = 10
S = 0

for i in range(N):
	S = S + a
	a = a + d

print(S)



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Torodd)
# Oppgave 3 b) Sparekonto - Påstand: Dobbelt så lang tid 

from sympy import Symbol, Eq, Reals, solveset

# Påstand
# Det vil gå nøyaktig dobbelt så lang tid før beløpet Per har på konto,
# har dobla seg, som det vil gå før beløpet Kåre har på konto, har dobla seg.
#
# Argumenter for at påstanden ikkje er riktig.
# Dersom dette stemmer må 1.03^2x og 1.06^x vera det same.
# 1.03^2x = (1.03^2)^x = 1.0609^x ≠ 1.06^x
#
# Påstanden er ikkje riktig.



# 🚀 programmering.no | 🤓 matematikk.as
# - Finn det største elementet i en liste med for-løkke

# Verdier
liste = [2, 3, 400, 50, 10]

# Funksjon for å finne max el i en liste med for-løkke
def maksimum_for_loop(liste):

	# Sett første el som forløpig max
	max = liste[0]

	# Itererer gjennom listen
	for i in range(1, len(liste)):

		# Oppdater max hvis el er større enn nåværende max
		if liste[i] > max:
			max = liste[i]

	return max

# Maksimum
max = maksimum_for_loop(liste)

# Print
print(f"Liste             : {liste}")
print(f"Maksimum i listen : {max}")



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (Matematikk AS)
# Oppgave 4 c) Aritmetisk- og geometrisk rekke - Uker før tilbud 2 til sammen gir mer lønn enn tilbud 1 

lonn_0      = 100       # Ukelønn første uke
a_n         = lonn_0    # Tilbud 1: Ukelønn første uke
a_d         = 10        # Tilbud 1: Aritmetisk økning (differanse)
a_s         = 0         # Tilbud 1: Aritmetisk sum
g_n         = lonn_0    # Tilbud 2: Ukelønn første uke
g_k         = 1.05      # Tilbud 2: Geometrisk økning (kvotient)
g_s         = 0         # Tilbud 2: Geometrisk sum
uker        = 1         # Antall uker
desimal     = 2         # Antall desimaler i avrunding

# Print-funksjon
def printing_rader(uker, tilbud, belop): print(f"Uke {uker} | Tilbud {tilbud} | Beløp: {round(belop, desimal)}")

# Oppgave c)
print("Oppg c)")

# Kjøres så lenge geometrisk ukelønn er lavere enn aritmetisk
while g_s <= a_s:
    a_s  += a_n  # Aritmetisk sum
    a_n  += a_d  # Aritmetisk formel
    g_s  += g_n  # Geometrisk sum
    g_n  *= g_k  # Geometrisk formel
    printing_rader(uker, "1 (A)", a_s)
    printing_rader(uker, "2 (G)", g_s)
    print("")
    uker += 1    # Uker øker med 1

# Trekker fra 1 uke
uker -= 1

# Print svar
print(f"Etter {uker} uker er den geometrisk summen størst ({round(g_s, desimal)} > {a_s})")



# 🚀 programmering.no | 🤓 matematikk.as
# - Eksempel på range (intervall) med kun øvre grense

# Variabler
intervall = range(10)

# Merk at øvre grense 10 ikke er inkludert
for tall in intervall:
    print(tall)



# Versjon: 0.0.166

# '''
# numpy()
from numpy import(
    polyfit,
    )

# sympy()
from sympy import (
    ConditionSet,
    core,
    diff,
    Eq,
    FiniteSet,
    Intersection,
    ln,
    log,
    nsolve,
    Reals,
    solve,
    solveset,
    Symbol,
    )

# deriver()
from ._exports.def_deriver_mas import (
    deriver,

    # Alias > 1
    derivert,
    momentan_vekst,
    momentan_vekstfart,
    )

# vekstfaktor_cas()
from ._exports.def_vekstfaktor_cas import (
    vekstfaktor_cas,
    )

# reggis()
from ._exports.def_reggis_matematikk_mas import (
    reggis,

    # Alias > 1
    reggis_cas,
    regresjon,
    regresjon_cas,
    regresjon_polynom,
    regresjon_polynom_cas,

    # Alias > 2
    cas_regresjon,
    cas_regresjon_polynom,
    regresjon_polynom_cas,
    )

# superlos()
from ._exports.def_superlos_matematikk_mas import (
    superløs,

    # Alias > 1
    los,
    losning,
    løs,
    løsning,
    superlos,
    super_los,
    super_løs,

    # Alias > 2
    los_super,
    løs_super,
    )

# ekstremalpunkt_max()
from ._exports.def_ekstremalpunkt_max_mas import (
    ekstremalpunkt_max,

    # Alias > 1
    ekstremalpunkt_maks,
    ekstremalpunkt_maksimalt,
    toppunkt,

    # Alias > 2 > ...
    )

# overskudd_max()
from ._exports.enhet_fra_overskudd_max import (
    enhet_fra_overskudd_max,

    # Alias > 1
    enhet_fra_overskudd_maks          = enhet_fra_overskudd_max,
    enhet_fra_overskudd_maksimalt     = enhet_fra_overskudd_max,
    enhet_fra_overskudd_mest          = enhet_fra_overskudd_max,
    enhet_fra_overskudd_storst        = enhet_fra_overskudd_max,
    enhet_fra_overskudd_størst        = enhet_fra_overskudd_max,

    # Alias > 2
    enhet_fra_maks_overskudd          = enhet_fra_overskudd_max,
    enhet_fra_maksimalt_overskudd     = enhet_fra_overskudd_max,
    enhet_fra_mest_overskudd          = enhet_fra_overskudd_max,
    enhet_fra_storst_overskudd        = enhet_fra_overskudd_max,
    enhet_fra_størst_overskudd        = enhet_fra_overskudd_max,
    )

# enhet_fra_overskudd_max()
from ._exports.enhet_fra_overskudd_max_med_kostnad import (
    enhet_fra_overskudd_max_med_kostnad,

    # Alias > 1 ...
    enhet_fra_overskudd_max_kostnad     = enhet_fra_overskudd_max_med_kostnad,

    # Alias > 2 > ...
    enhet_fra_max_overskudd_med_kostnad = enhet_fra_overskudd_max_med_kostnad,
    enhet_fra_max_overskudd_kostnad     = enhet_fra_overskudd_max_med_kostnad,
    )

# enhet_og_pris_fra_inntekt_max()
from ._exports.def_enhet_og_pris_fra_inntekt_max_mas import (
    enhet_og_pris_fra_inntekt_max,

    # Alias > 1
    enhet_og_pris_fra_inntekt_max,

    # Alias > 2 > ...
    )

# '''



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Torodd)
# Oppgave 2 a) Venstrehendte - Minst 25 venstrehendte gutter 

import numpy as np

# definerer tilfeldighetsgeneratoren
rng = np.random.default_rng()

# tal simuleringar
N = 10000000

# simulerer uttrekk av 280 gutar
simulert = rng.binomial(n = 280, p = 0.1, size = N)

# tel opp gunstige utfall og finn sannsynet
gunstige = sum(simulert >= 25)
sannsyn = gunstige / N

print(f"P(X>=25) = {sannsyn:.4f}")



# 🚀 programmering.no | 🤓 matematikk.as
# - Kopier liste med extend()

# Verdier
liste = ["A", "B", "C"]

# Funksjon som kopierer en liste med extend
def copy_liste_extend(liste):

    # Nullstill
    liste_copy = list()

    # Extend
    liste_copy.extend(liste)

    return liste_copy

# Kopiert liste
liste_copy = copy_liste_extend(liste)

# Print
print(f"Original liste : {liste}")
print(f"Kopiert liste  : {liste_copy}")



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 1 a) Sofa-produksjon til møbelfabrikk - Finn O(x) 
# - Lager polynom-funksjon av valgfri grad vha. regresjon (CAS)
# - Kopier denne kommandoen i terminalen for å importere matematikk (CAS)
#   $ pip install matematikk

from matematikk import reggis, Symbol

# a) Finn O(x)
K = reggis(variabel = Symbol("x"),
           grad     = 2,
           x_liste  = [10, 25, 40, 70, 100, 140, 180],
           y_liste  = [270, 550, 870, 1500, 2200, 3300, 4500],
           rund     = 2)

print(K)




# 🚀 programmering.no | 🤓 matematikk.as
# - Type casting fra string til integer

# Variabler
tall_1 = 60
tall_2 = "40"

# For å regne med tall_2, så må tallet type castes fra str til int
sum = tall_1 + int(tall_2)

# Print
print(sum)



# 🚀 programmering.no | 🤓 matematikk.as
# - Finn produktet av alle elementene i en liste med math

import math

# Input
liste = input("Liste: ") # Husk [ ] og "", f.eks. [2.5, 10]

# Enkel funksjon som klipper opp inp-str til liste med float-el
def str_til_float_liste(str_liste = list()):

    float_liste = list()
    el = str()
    for i in range(len(str_liste)):
        if str_liste[i] == " ": pass
        elif str_liste[i] == "[": pass
        elif str_liste[i] == "]":
            float_liste.append(float(el))
            el = ""
        elif str_liste[i] == ",":
            float_liste.append(float(el))
            el = ""
        else: el += str_liste[i]
    return float_liste

# Input-string endres til liste med float-el
liste = str_til_float_liste(liste)

# Finn produktet med math
produkt = math.prod(liste)

# Print
print(f"Liste                   : {liste}")
print(f"Produktet av elementene : {produkt}")



# 🚀 programmering.no | 🤓 matematikk.as
# Sinus 1T (2020) - BrettBoka - 1.5 Formler (s. 27) 
# Oppgave 1.59 Vanjas scooter (Matematikk AS)

# Blokk med definisjoner
_blokk             = 1 # 0: Av, 1: På, Skjul/debug deler av koden med if-blokker
if _blokk == 1:

    # Konstanter
    distanse_igjen     = 14            # Distanse hun har igjen, [km]
    distanse_kjort     = 0             # Distanse hun har kjørt, [km]
    distanse_tot       = 35            # Distanse mellom husene, [km]
    distanse_0         = 0             # Distanse hjemme er 0, [km]
    tid_igjen          = 0             # Tiden hun har igjen, [h]
    tid_kjort          = 0             # Tiden hun har kjørt, [h]
    tid_tot            = 0             # Tiden hun bruker mellom husene, [h]
    tid_min_per_time   = 60            # 60 min per time
    fart               = 0             # Farten hennes, [km/h], v = s / t <=> t = s / v
    fart_km_per_min    = 0.7           # 0.7 km per min

    # Konverter fart fra km/min -> km/h
    fart               = fart_km_per_min * tid_min_per_time

    # Enheter
    enhet_km           = "km"          # Kilometer
    enhet_min          = "min"         # Minutter
    enhet_km_h         = "km/h"        # Kilometer i timen

    # Tekst
    txt_oppg           = "Oppg 1.55"
    txt_strek          = "---------"
    txt_info_1         = "Distanse mellom husene            :"
    txt_a_1            = "a) Formel for tiden hun har kjørt"
    txt_a_2            = "      s = 35 - 0.7 t"
    txt_a_3            = "      0.7 t = 35 - s"
    txt_a_4            = "      0.7 t / 0.7 = (35 - s) / 0.7"
    txt_a_5            = "      t = (35 - s) / 0.7"
    txt_b              = "b) Tiden hun har kjørt            :"
    txt_c              = "c) Tiden hun bruker mellom husene :"
    txt_info_2         = "Tiden hun har igjen               :"
    txt_info_3         = "Distanse hun har kjørt            :"
    txt_info_4         = "Distanse hun har igjen            :"
    txt_info_5         = "Farten hennes                     :"

    # Innstillinger
    inp                = 1             # # 0: Boken, 1: Bruker
    avrund_valg        = 1             # # 0: Ikke avrunding, 1: Avrunding
    avrund_ant         = None          # # None: 12.78 -> 13, 0: 12.78 -> 13.0, 1: 12.78 -> 12.8, 2: 12.78 -> 12.78

# Input
print("")
print(txt_strek)
print(txt_oppg)
print(txt_strek)
print("")
print(txt_info_1, distanse_tot, enhet_km)
if inp == 1:

    print("")
    distanse_igjen = input("Hvor langt har hun kjørt [km]? ")
    fart = input("Hva er farten hennes [km/h]? ")

    # Type casting: str -> float
    distanse_igjen     = float(distanse_igjen)
    fart               = float(fart)

# a) Formel for tiden hun har kjørt
def get_tid(distanse):
    # v = s / t <=> t = s / v
    _tid           = (distanse_tot - distanse) / fart
    _tid           = _tid * tid_min_per_time
    return _tid

def get_tid_ez(distanse):
    # t = (35 - s) / 0.7
    _tid           = (distanse_tot - distanse) / fart_km_per_min
    return _tid

# b) Tiden hun har kjørt
tid_kjort          = get_tid(distanse_igjen)

# c) Tiden hun bruker mellom husene
tid_tot            = get_tid(distanse_0)

# Tiden hun har igjen
tid_igjen          = tid_tot - tid_kjort

# Distanse hun har kjørt
distanse_kjort     = distanse_tot - distanse_igjen

# Avrunding
if avrund_valg == 1:
    tid_kjort      = round(tid_kjort, avrund_ant)
    tid_tot        = round(tid_tot, avrund_ant)
    tid_igjen      = round(tid_igjen, avrund_ant)
    distanse_kjort = round(distanse_kjort, avrund_ant)
    distanse_igjen = round(distanse_igjen, avrund_ant)
    fart           = round(fart, avrund_ant)

# Print
print("")
print(txt_a_1)
print(txt_a_2)
print(txt_a_3)
print(txt_a_4)
print(txt_a_5)
print(txt_b, tid_kjort, enhet_min)
print(txt_c, tid_tot, enhet_min)
print("")
print(txt_info_2, tid_igjen, enhet_min)
print(txt_info_3, distanse_kjort, enhet_km)
print(txt_info_4, distanse_igjen, enhet_km)
print(txt_info_5, fart, enhet_km_h)
print("")



# 🚀 programmering.no | 🤓 matematikk.as
# S1 - Eksamen - 2023 Høst (Matematikk AS)
# Oppgave 1 a) Sofa-produksjon til møbelfabrikk - Finn O(x) 
# - Lager polynom-funksjon av valgfri grad vha. regresjon (string)
# - Importer reggis() fra matematikk for å regne med polynomet i CAS
# - Kopier kommandoene i terminalen og importer pakkene (CAS)
#   $ pip install numpy
#   $ pip install matematikk

from numpy import polyfit

# Variabler
variabel           = "x"           # Polynomets variabel
navn               = "K"           # Polynomets navn (vs)
grad               = 2             # Polynomets grad
koeff_liste        = list()        # Liste med koeffesienter i fra polyfit()
koeff              = float()       # Polynomets koeffisienter
ledd               = str()         # Polynomets ledd
ledd_grad          = int()         # Polynomets ledd inkludert grad
uttrykk            = str()         # Polynomets uttrykk (hs)
poly               = str()         # Polynomet (vs og hs)
x_liste            = [             # Liste med x-verdier
    10,
    25,
    40,
    70,
    100,
    140,
    180]
y_liste            = [             # Liste med y-verdier
    270,
    550,
    870,
    1500,
    2200,
    3300,
    4500]
rund               = 2             # -1: Ingen avrunding, 0: Eks: 123.0, n > 0: n siffer etter desimaltegnet, None: Heltall
_blokk             = 1             # 0: Av, 1: På, Skjul/debug deler av koden med if-blokker

# a) Finn O(x)
if _blokk == 1:

    # polyfit() returnerer en liste med regresjons-koeffesientene
    koeff_liste = polyfit(x_liste, y_liste, grad)

    # Hver iterasjon lager det neste leddet i polynomet
    for i in range(len(koeff_liste)):

        # Runder av koeffesienten
        if rund == -1: koeff = koeff_liste[i]
        if rund != -1: koeff = round(koeff_liste[i], rund)

        # Type caster float -> str
        koeff = str(koeff)

        # Lager leddene i fra størst til minst grad
        ledd_grad = grad - i

        # Formaterer leddene
        if ledd_grad >= 2: ledd = koeff + f"{variabel}^{str(ledd_grad)} + "
        if ledd_grad == 1: ledd = koeff + f"{variabel} + "
        if ledd_grad == 0: ledd = koeff

        # Legger det nye leddet til polynomet
        uttrykk += ledd

    poly = navn + f"({variabel}) = " + uttrykk

    # Svar-setninger
    svar_a_liste = list()
    svar_a_liste.append(f"")
    svar_a_liste.append(f"Oppg a)")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"- Med regresjon fra numpy (polyfit) ser vi at hvis bedriften")
    svar_a_liste.append(f"  produserer x enheter, så vil")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"    {poly}")
    svar_a_liste.append(f"")
    svar_a_liste.append(f"  være en god model for det måntlige overskuffet (i tusen kroner)")

    # Print svar-setninger
    for svar in svar_a_liste: print(svar)



# 🚀 programmering.no | 🤓 matematikk.as
# - Del en liste opp i mindre lister med n elementer i hver liste

# Verdier
steg  = 4
liste = [
                    "a", "b", "c", "d", "e", "f", "g", "h", "i", "j",
                    "k", "l", "m", "n", "o", "p", "q", "r", "s", "s",
                    "u", "v", "w", "x", "y", "z"
                ]

# Del opp listen i del-lister med n elementer i hver del-liste
def oppdeling_lister_for_loop(liste):

    # Nullstill
    start      = 0
    slutt      = len(liste)
    del_lister = []

    # Itererer gjennom listen
    for i in range(start, slutt, steg):
        del_lister.append(liste[i : i + steg])

    return del_lister

# Del-lister
del_lister = oppdeling_lister_for_loop(liste)

# Print
print(f"Del-lister (hele): {del_lister}")
print("Del-lister (en og en):")
for del_liste in del_lister:
	print(f"{del_liste}")



from matematikk import diff

def deriver(uttrykk, vari):

	uttrykk_derivert = diff(uttrykk, vari) 

	return uttrykk_derivert

# Alias > 1
derivert                        = diff
momentan_vekst                  = diff
momentan_vekstfart              = diff

# Alias > 2 > ...



# 🚀 programmering.no | 🤓 matematikk.as
# - Reverserer rekkefølgen til elementene i en liste
# - Hastighet reverse(): Ca. 33 ms per loop for liste m/ 100 000 000 el

# Input
liste = input("Listen som reverseres: ") # Husk [ ], f.eks. [1, 2, 3, 4]

# Enkel funksjon som klipper opp inp-str til liste med int-el
def str_til_int_liste(str_liste = list()):

    int_liste = list()
    el = str()
    for i in range(len(str_liste)):
        if str_liste[i] == " ": pass
        elif str_liste[i] == "[": pass
        elif str_liste[i] == "]":
            int_liste.append(int(el))
            el = ""
        elif str_liste[i] == ",":
            int_liste.append(int(el))
            el = ""
        else: el += str_liste[i]
    return int_liste

# Input-string endres til liste med int-el
liste = str_til_int_liste(liste)

# Kopier listen hvis du vil beholde originalen
reversert_liste = liste.copy()

# Reverse
reversert_liste.reverse()

# Print
print(f"Original liste  : {liste}")
print(f"Reversert liste : {reversert_liste}")



# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (NDLA)
# Oppgave 3 a) Coca-Cola vs. Pepsi-Cola - 10 glass 

from scipy.stats import binom

n = 10      # antall forsøk
p = 0.5     # sannsynligheten
X = []      # ei liste for de ulike verdiene X kan ha

for i in range(n+1):
    X.append(i) # legger inn alle de mulige verdiene til X
rettesvar = binom.pmf(X,n,p) # lager en array med sannsynlighetsfordelingen
print("ant :: ", len(X))
print(f"Sannsynligheten for 6 rette svar er {rettesvar[6]:.3f}.") # printer ut svaret



# 🚀 programmering.no | 🤓 matematikk.as

import matematikk as mt

def enhet_fra_overskudd_max(variabel = mt.Symbol(""),
                            uttrykk  = mt.Symbol(""),
                            desimal  = None,
                            debug    = -1):

    # overskudd_max() er en undergruppe av ekstremalpunkt_max()
    variabel_max = mt.ekstremalpunkt_max(variabel = variabel,
                                         uttrykk  = uttrykk,
                                         rund     = desimal,
                                         debug    = debug)

    return variabel_max

# Alias > 1
enhet_fra_overskudd_maks          = enhet_fra_overskudd_max
enhet_fra_overskudd_maksimalt     = enhet_fra_overskudd_max
enhet_fra_overskudd_mest          = enhet_fra_overskudd_max
enhet_fra_overskudd_storst        = enhet_fra_overskudd_max
enhet_fra_overskudd_størst        = enhet_fra_overskudd_max

# Alias > 2
enhet_fra_maks_overskudd          = enhet_fra_overskudd_max
enhet_fra_maksimalt_overskudd     = enhet_fra_overskudd_max
enhet_fra_mest_overskudd          = enhet_fra_overskudd_max
enhet_fra_storst_overskudd        = enhet_fra_overskudd_max
enhet_fra_størst_overskudd        = enhet_fra_overskudd_max




# 🚀 programmering.no | 🤓 matematikk.as
# S2 - Eksamen - 2023 Vår (NDLA)
# Oppgave 4 a) Aritmetisk- og geometrisk rekke - Ukentlig beløp de 4 første ukene 

an = 100                # setter startverdi for ukelønna
print("Tilbud 1:")

for i in range(4):      # for-løkke som kjører 4 ganger
    print(an)           # skriver ut ukelønna
    an = an + 10        # regner ut ukelønn for neste uke

bn = 100                # setter startverdi for ukelønna
print("Tilbud 2:")
for i in range(4):      # for-løkke som kjører 4 ganger
    print(bn)           # skriver ut ukelønna
    bn = bn*1.05        # regner ut ukelønn for neste uke



# 🚀 programmering.no | 🤓 matematikk.as
# - Finn antall bokstaver (char) med len()

# Variabler
hei      = "Hei, Python 🐍"
char_ant = len(hei)

# Print
print(f"String: {hei}")
print(f"Antall chars: {char_ant}")
